<?php

namespace Drp\SimpleCms\Contracts;

use Drp\SimpleCms\Content;
use Illuminate\Database\Eloquent\Relations\HasMany;

interface Publishable
{
    const STATUS_DRAFT = 'Draft';

    const STATUS_REVIEW = 'In review';

    const STATUS_PUBLISHED = 'Published';

    const STATUS_UNPUBLISHED = 'Un-published';

    const STATUSES = [
        self::STATUS_DRAFT,
        self::STATUS_REVIEW,
        self::STATUS_PUBLISHED,
        self::STATUS_UNPUBLISHED,
    ];

    public function generateContentId(): int;

    public function isPublished(): bool;

    public function isUnPublished(): bool;

    public function isDraft(): bool;

    public function isInReview(): bool;

    public function isDraftOrInReview(): bool;

    public function updatePublishStatus(): void;

    public function updateReviewStatus(): void;

    public function canBePublished(): bool;

    public function canBeReviewed(): bool;

    public function canBeReset(): bool;

    public function hasPublishedVersion(): bool;

    public function hasDraftOrReviewVersion(): bool;

    public function hasUnPublishedVersion(): bool;

    /**
     * Remove the draft version of the content so there is only the published version left.
     *
     * @return mixed
     */
    public function resetDraft();

    public function allVersionsOfContent(): HasMany;

    public static function getLatestVersionById(int $contentId): Content;

    public function getPublishedVersion(): ?Publishable;

    public function getUnpublishedVersion(): ?Publishable;

    public function getDraftOrReviewVersion(): ?Publishable;

    public function beforeSave(): ?\Closure;

    public function afterSave(): ?\Closure;

    /**
     * @return mixed
     */
    public static function getAllVersionsPrioritisedByStatus(?array $with = []);

    public function isContentDirty(array $attributes): bool;
}
