<?php

namespace Drp\SimpleCms\Http\Requests;

use Drp\SimpleCms\Contracts\Publishable;
use Illuminate\Foundation\Http\FormRequest;

class ContentRequest extends FormRequest implements \Drp\SimpleCms\Contracts\ContentRequest
{
    /**
     * @var array
     */
    protected $rules = [];

    /**
     * @var array
     */
    protected $messages = [];

    /**
     * Prepare the input using the given content.
     * Retrieves the before updated closure from the content
     * and calls it using the current request.
     *
     * @return array|string
     */
    public function prepareInput(Publishable $content)
    {
        $this->prepareValidation($content);

        $callback = $content->getCallbackForUpdateRequest($this);

        if ($callback instanceof \Closure) {
            call_user_func($callback, $this);
        }
        return $this->input();
    }

    public function prepareValidation(Publishable $content): void
    {
        $this->rules = $content->getSaveRules();
        $this->messages = $content->getSaveMessages();
    }

    public function authorize(): bool
    {
        return true;
    }

    public function rules(): array
    {
        return $this->rules;
    }

    public function messages(): array
    {
        return $this->messages;
    }
}
